#!/usr/bin/python
# -*- coding: UTF-8 -*-

############################################################################
#                                                                          #
#  Utilitaire de configuration de jobs pour bacula                         #
#  Crée un fichier /etc/bacula/eolejobdefs.pic                             #
#  contenant le dictionnaire des jobs                                      #
#  de ce dictionnaire est généré le fichier /etc/bacula/eoleschedule.conf  #
#                                                                          #
#  licence GPL Eole                                                        #
#                                                                          #
#  Klaas TJEBBES klaas.tjebbes@ac-dijon.fr                                 # 
#                                                                          #
############################################################################

import pickle
import os
import sys
import getopt

def chargedico():
    """charge le dictionaire contenant la liste des jobs définis précédemment
    c'est un dictionnaire de dictionnaires :
        dicosched = {1: dicojob1, 2: dicojob2 ... }
    """
    try:
        dicosched = pickle.load(file('/etc/bacula/eolejobdefs.pic','r'))
    except:
        dicosched = {}
    return dicosched

def savedico(dicosched):
    """enregistre le dictionnaire avec le nouveau contenu
    en réorganisant les "keys" du dictionnaire
    """
    tmpdicosched = {}
    cle = 1
    #trie du dicosched : si un job a été enlevé au milieu les clés restent consécutives
    for i in dicosched.keys():
        tmpdicosched[cle]=dicosched[i]
        cle += 1
    pickle.dump(tmpdicosched,file('/etc/bacula/eolejobdefs.pic','w'))
    return 0

def createdicojob(niveau,typesav,heure,jourdebut,jourfin=None):
    """dictionnaire contenant un job
    dicojob = {'niveau': 'Level=Full/Incremental' , 'typesav': 'Complete/Restreinte' , 'heure': 'XXh00' , 'jourdebut': 'mon/tue...' , 'jourfin': 'mon/tue...'/None}
    """
    dicojob = {}
    dicojob['niveau'] = 'Level='+niveau
    heure = heure.strip()
    if len(heure.split(':')[0])==1:
        heure = '0'+heure
    dicojob['heure'] = heure
    dicojob['jourdebut'] = jourdebut
    dicojob['jourfin'] = jourfin
    dicojob['typesav'] = typesav
    return dicojob

def ecrirejobs(dicosched):
    """écrit les fichiers /etc/bacula/eoleschedule.conf et /etc/bacula/eoleschedulerestreint.conf à partir du dictionnaire de jobs
    """
    montemplate1 = """Run = %(niveau)s %(jourdebut)s-%(jourfin)s at %(heure)s\n"""
    montemplate2 = """Run = %(niveau)s %(jourdebut)s at %(heure)s\n"""
    listejobs = []
    listejobsrestreint = []
    for job in dicosched.keys():
        if dicosched[job]['typesav'] == 'Complete':
            if dicosched[job]['jourfin']:
                listejobs.append(montemplate1 % dicosched[job])
            else:
                listejobs.append(montemplate2 % dicosched[job])
        else:
            if dicosched[job]['jourfin']:
                listejobsrestreint.append(montemplate1 % dicosched[job])
            else:
                listejobsrestreint.append(montemplate2 % dicosched[job])
    eoleschedule = file('/etc/bacula/eoleschedule.conf','w')
    for i in listejobs:
        eoleschedule.write(i)
    eoleschedule.close()
    eoleschedulerestreint = file('/etc/bacula/eoleschedulerestreint.conf','w')
    for i in listejobsrestreint:
        eoleschedulerestreint.write(i)
    eoleschedulerestreint.close()
    os.system('/etc/init.d/bacula-director restart')
    return 0

def listerjobs():
    """affiche la liste des jobs définis en traduisant les jours
    """
    montemplate1 = """Sauvegarde %(niveau)s du %(jourdebut)s au %(jourfin)s à %(heure)s"""
    montemplate2 = """Sauvegarde %(niveau)s le %(jourdebut)s à %(heure)s"""
    dicosched = chargedico()
    dicotraduc = { 'mon':'lundi', 'tue':'mardi', 'wed':'mercredi', 'thu':'jeudi', 'fri':'vendredi', 'sat':'samedi', 'sun':'dimanche', 'Level=Incremental':'Incrémentale', 'Level=Full':'Totale' }
    for job in dicosched.keys():
        dicosched[job]['jourdebut']=dicotraduc[dicosched[job]['jourdebut']]
        dicosched[job]['niveau']=dicotraduc[dicosched[job]['niveau']]
        if dicosched[job]['jourfin']:
            dicosched[job]['jourfin']=dicotraduc[dicosched[job]['jourfin']]
            print str(job)+" : "+montemplate1 % dicosched[job]
        else:
            print str(job)+" : "+montemplate2 % dicosched[job]
    sys.exit()
    
def supprimjob(suppr):
    """supprime le job X
    on obtient X à partir de l'affichage des jobs
    """
    try:
        dicosched = chargedico()
        del(dicosched[int(suppr)])
        savedico(dicosched)
        ecrirejobs(dicosched)
    except:
        print "La sauvegarde N°"+suppr+" n'existe pas"
    return 0

def main():
    """charge les options et définit l'action à effectuer (création d'un job, suppression ou affichage de la liste des jobs)
    """
    # gestion des options
    ajout = None
    jourdebut = None
    jourfin = None
    heure = None
    niveau = None
    liste = None
    suppr = None
                
    try:
        options, args = getopt.getopt(sys.argv[1:],'d:f:h:t:n:s:l',
        ['jourdebut=',
        'jourfin=',
        'heure=',
        'niveau=',
        'typesav=',
        'suppr=',
        'liste'])

    except getopt.GetoptError:
        er_type, er_value, er_traceback = sys.exc_info()
        er_msg = str(er_value)
        sys.exit("l'option -%s est inconnue ou nécessite un argument" % er_msg[er_msg.index('-')+1])

    for (opt,val) in options:
        # premier jour
        if (opt == '-d') or (opt == '--jourdebut'):
            jourdebut = val
        # dernier jour si absent premier jour deviendra jour unique
        if (opt == '-f') or (opt == '--jourfin'):
            jourfin = val
        # à quelle heure
        if (opt == '-h') or (opt == '--heure'):
            heure = val
        # Full ou Incremental
        if (opt == '-n') or (opt == '--niveau'):
            niveau = val.capitalize()
        if (opt == '-t') or (opt == '--typesav'):
            typesav = val.capitalize()
        # suppression
        elif (opt == '-s') or (opt == '--suppr'):
            suppr=val
        # ou alors on liste
        elif (opt == '-l') or (opt == '--liste'):
            listerjobs()
        
    # supprime un job
    if suppr:
        supprimjob(suppr)
        return 0

    # un nouveau job
    elif jourdebut and heure and niveau:
        dicosched = chargedico()
        nbjob = len(dicosched)
        dicosched[nbjob+1] = createdicojob(niveau,typesav,heure,jourdebut,jourfin)
        savedico(dicosched)
        ecrirejobs(dicosched)
        return 0

    else:
        sys.exit("""
 options possibles :
    -d (--jourdebut)   : premier jour de la sauvegarde / jour de la sauvegarde (mon,tue,wed,thu,fri,sat,sun)
    -f (--jourfin)     : dernier jour de la sauvegarde (facultatif)
    -h (--heure)       : l'heure de la sauvegarde (hh:mm)
    -n (--niveau)      : Full / Incremental
    -t (--typesav)     : Complete / Restreinte

    -l (--liste)       : liste les sauvegardes programmées

    -s (--suppr)     N : supprime la sauvegarde programmée N

 Exemples :
    baculaeole.py -d sun -h 1:05 -t Full
    baculaeole.py -d mon -f sat -h 1:05 -n Incremental -t Complete
    baculaeole.py -l
    baculaeole.py -s 1
    """)

if __name__ == "__main__":
    main()

