#!/bin/bash

# Ces variables doivent être identiques à celles
# dans la section "__init__(self)" de
# /etc/bacula/scripts/DirStartUp.py
REPERTOIRE='/var/sauvegardes'
VERSION=`cat /etc/eole/version`
TEMOIN='/var/www/ead/tmp/temoin-bacula'
EADRESULT='/var/www/ead/tmp/resultat-bacula'
EADRAPPORT='/var/www/ead/tmp/rapport-bacula.txt'

# pour que l'affichage de [ ok ] soit ok
export TERM='dumb'


. ParseDico


testcmd(){
    [ "$1" -ne "0" ] && exit 1
}

servicestop(){
    [ -e /etc/init.d/samba ] &&  /etc/init.d/samba stop
    [ -e /etc/init.d/slapd ] &&  /etc/init.d/slapd stop
    if [ "$mysql" != "non" ]
    then
	    [ -e /etc/init.d/mysql ] &&  /etc/init.d/mysql stop
    fi
    [ -e /etc/init.d/sympa ] &&  /etc/init.d/sympa stop
}

servicestart(){
    [ -e /etc/init.d/samba ] &&  /etc/init.d/samba start
    # LDAP redémarré avant la sauvegarde
    #[ -e /etc/init.d/slapd ] &&  /etc/init.d/slapd start
    if [ "$mysql" != "non" ]
    then
	    [ -e /etc/init.d/mysql ] &&  /etc/init.d/mysql start
    fi
    [ -e /etc/init.d/sympa ] &&  /etc/init.d/sympa start
    # parce qu'il ne suppporte pas l'arrêt de LDAP ...
    if [ "$sso" != "non" ]
    then
	    [ -e /etc/init.d/eole-sso ] &&  /etc/init.d/eole-sso restart
    fi
}

##############
# Sauvegarde #
##############
## LDAP ##
prebackupldap(){
    /etc/init.d/slapd stop
    # Sauvegarde LDAP, LDAP fonctionnant pendant la sauvegarde l'accélère
    rm -rf /home/sauv_ldap
    rm -rf /home/sauv_ldap.ldif
    DF=`df -Pm /home/|tail -n1|awk '{print $4}'`
    DU=`du -sm /var/lib/ldap/|awk '{print $1}'`
    if [ "$DF" -gt "$DU" ]
    then
	    slapcat -v > /home/sauv_ldap.ldif
    else
	    echo "Espace insuffisant sur /home ($DF Mo) pour sauvegarder LDAP, Abandon."
	    exit 1
    fi
    /etc/init.d/slapd start
}

postbackupldap(){
    # Suppression de la sauvegarde LDAP
    rm -rf /home/sauv_ldap.ldif
}

## MySQL ##
prebackupmysql(){
    if [ "$mysql" == "non" ]
    then
	    exit 0
    fi
    rm -f /home/sauv_mysql.sql
    /etc/init.d/mysql stop
    sleep 3
    /etc/init.d/mysql start
    i=0
    while [ ! -e /var/run/mysqld/mysqld.sock ]
    do
        let i=$i+1
        if [ "$i" -eq "30" ]
        then
                break
        fi
        sleep 1
    done
    mysqldump --defaults-file=/etc/mysql/debian.cnf --all-databases --create-options -Q -c --lock-tables > /home/sauv_mysql.sql
}

postbackupmysql(){
    if [ "$mysql" == "non" ]
    then
	    exit 0
    fi
    # Suppression de la sauvegarde MySQL
    rm -rf /home/sauv_mysql.sql
}

## Fichiers ##
prebackup(){
    touch $EADRAPPORT
    chmod 644 $EADRAPPORT
    touch $TEMOIN

    # Montage du support de sauvegarde
    /usr/share/eole/bacula/montage.sh pre
    testcmd $?

    # Commandes perso
    [ -e /etc/bacula/precommandes ] && sh /etc/bacula/precommandes

    #Sauvegarde du SID
    if [ $VERSION == horus-2.0 -o "$VERSION" == "horus-2.1" ]
    then
        net getlocalsid | awk -F: '{print $2}' | awk -F " " '{print $1}'>/etc/eole/HORUS_SID
    elif [ $VERSION == scribe-2.0 -o "$VERSION" == "scribe-2.1" ]
    then
        net getlocalsid | awk -F: '{print $2}' | awk -F " " '{print $1}'>/etc/eole/SCRIBE_SID
    fi

    # Arrêt des services
    servicestop

    # Redémarrage de LDAP
    [ -e /etc/init.d/slapd ] &&  /etc/init.d/slapd start

    exit 0
}

postbackup(){
    # Suppression du fichier contenant le SID
    if [ $VERSION == horus-2.0 -o "$VERSION" == "horus-2.1" ]
    then
        rm -rf /etc/eole/HORUS_SID
    elif [ $VERSION == scribe-2.0 -o "$VERSION" == "scribe-2.1" ]
    then
        rm -rf /etc/eole/SCRIBE_SID
    fi
    # Suppression de la sauvegarde LDAP
    rm -rf /home/sauv_ldap.ldif
    # Sauvegarde de la configuration du serveur
    cp /etc/eole/config.eol $REPERTOIRE/$VERSION-config.eol -f

    # Commandes perso
    [ -e /etc/bacula/postcommandes ] && sh /etc/bacula/postcommandes
    # Redémarrage des services
    servicestart
    rm $TEMOIN -f
    echo "Dernière sauvegarde effectuée le `date`." > $EADRESULT
    chmod 644 $EADRESULT
    /usr/share/eole/bacula/montage.sh post
    exit 0
}

postbackuperr(){
    # En cas de plantage de la sauvegarde
    rm $TEMOIN -f
    echo "La sauvegarde a échoué `date`." > $EADRESULT
    chmod 644 $EADRESULT
    # Redémarrage des services
    servicestart
    # Démontage
    /usr/share/eole/bacula/montage.sh post
    exit 0
}

## Catalog Bacula ##
precatalog(){
    # Montage du support de sauvegarde
    /usr/share/eole/bacula/montage.sh pre
    testcmd $?
    # Création et copie du .sql
    /etc/bacula/scripts/make_catalog_backup bacula bacula
    cp /var/lib/bacula/bacula.sql $REPERTOIRE/$VERSION-bacula.sql -f
}

postcatalog(){
    # Effacement du .sql
    /etc/bacula/scripts/delete_catalog_backup
    # Démontage du support
    /usr/share/eole/bacula/montage.sh post
}


################
# Restauration #
################
prerestoreall(){
    [ -e /etc/init.d/samba ] &&  /etc/init.d/samba stop
    [ -e /etc/init.d/mysql ] &&  /etc/init.d/mysql stop
    [ -e /etc/init.d/sympa ] &&  /etc/init.d/sympa stop
    exit 0
}

## LDAP ##
postrestoreldap(){
    # Restauration LDAP

    if [ -e /home/sauv_ldap.ldif ]
    then
        /etc/init.d/slapd stop
        rm -rf /var/lib/ldap/
	mkdir /var/lib/ldap/
        slapadd  -l /home/sauv_ldap.ldif
        chown openldap:openldap /var/lib/ldap/ -R
        rm -rf /home/sauv_ldap.ldif
        sleep 5
        /etc/init.d/slapd start
    fi
    echo
    echo "*** Restauration LDAP terminée ***"
    echo
}

## MySQL ##
postrestoremysql(){
    if [ "$mysql" == "non" ]
    then
	    exit 0
    fi
    /etc/init.d/mysql stop
    sleep 3
    /etc/init.d/mysql start
    sleep 3
    mysql --defaults-file=/etc/mysql/debian.cnf < /home/sauv_mysql.sql
    rm -rf /home/sauv_mysql.sql
}

## Fichiers ##
postrestore(){
    # Gestion du SID, 
    if [ "$VERSION" == "scribe-1.0" -o "$VERSION" == "scribe-2.0" -o "$VERSION" == "scribe-2.1" ]
    then
        if [ -f /etc/eole/SCRIBE_SID ]
        then
            cat /etc/eole/SCRIBE_SID | xargs net setlocalsid
        fi
	#les uidNumbers changent d'une version à l'autre
	chown sympa:sympa /etc/mail/sympa_aliases
	chown mysql:mysql /var/lib/mysql/ -R
    elif [ "$VERSION" == "horus-2.0" -o "$VERSION" == "horus-2.1" ]
    then
        if [ -f /etc/eole/HORUS_SID ]
        then
            cat /etc/eole/HORUS_SID | xargs net setlocalsid
        fi
    fi
    echo
    echo "*** Restauration terminée ***"
    echo
    exit 0
}

## Restauration complète ##
postrestoreall(){
    echo "################################"
    echo "# La restauration est terminée #"
    echo "#    Appuyez sur Ctrl+C et     #"
    echo "#   Lancez 'reconfigure' !!!   #"
    echo "################################"
}


case $1 in
  prebackupldap)
      prebackupldap
      ;;
  postbackupldap)
      postbackupldap
      ;;
  prebackupmysql)
      prebackupmysql
      ;;
  postbackupmysql)
      postbackupmysql
      ;;
  prebackup)
      prebackup
      ;;
  postbackup)
      postbackup
      ;;
  postbackuperr)
      postbackuperr
      ;;
  precatalog)
      precatalog
      ;;
  postcatalog)
      postcatalog
      ;;
  prerestoreall)
      prerestoreall
      ;;
  postrestoreldap)
      postrestoreldap
      ;;
  postrestoremysql)
      postrestoremysql
      ;;
  postrestore)
      postrestore
      ;;
  postrestoreall)
      postrestoreall
      ;;
  servicestart)
      servicestart
      ;;
  servicestop)
      servicestop
      ;;
*)
      echo "Argument $* inconnu"
      echo "Usage : "$0" prebackupldap | prebackupmysql | prebackup | postbackupldap |postbackupmysql |  postbackup | postbackuperr"
      echo "Usage : "$0" precatalog | postcatalog"
      echo "Usage : "$0" prerestoreall | postrestoreldap | postrestoremysql | postrestore | postrestoreall"
      echo "Usage : "$0" servicestart | servicestop"
      exit 1
esac

